/* main_srv.c
 * server using UNIX domain stream protocol
 */

#include "unix.h"
#include "sign.h"
#include <errno.h>


int main() {
  int sockfd,newsockfd,clilen,childpid,servlen,n;
  struct sockaddr_un cli_addr, serv_addr;
  char recvline[MAXLINE];

  if( (ignoresig(SIGCHLD) == SIG_ERR) ) {
      perror("Signal");
      exit(1);
  }

  /*
   * open a UNIX domain stream socket
   */
   if ((sockfd=socket(PF_UNIX,SOCK_STREAM,0)) < 0 ) {
     perror("server: can't open a stream socket");
     exit(2);
  }

  /*
   * bind our local address so that the client can send to us
   */

  /* set all with zeros */
  bzero( (char *) &serv_addr, sizeof(serv_addr));

  serv_addr.sun_family = AF_UNIX;
  strcpy(serv_addr.sun_path, UNIXSTR_PATH);

  /* determine length of address: */
  servlen = strlen(serv_addr.sun_path) + sizeof(serv_addr.sun_family);

  if (bind(sockfd, (struct sockaddr *) &serv_addr, servlen) < 0 ) {
    perror("server: can't bind local address");
    exit(3);
  }

  listen(sockfd,5);

  while(1) {
    /*
     * wait for a connection from a client process
     * - concurrent server -
     */

    clilen = sizeof(cli_addr);
    newsockfd = accept(sockfd, (struct sockaddr *) &cli_addr,
          &clilen);
    if (newsockfd < 0 ) {
      if (errno == EINTR)
	continue;
      perror("server: accept error");
      exit(4);
    }

    if ( (childpid = fork() ) < 0 ) {
      perror("server: can't fork");
      exit(5);
    }
    else if (childpid == 0) {  /* child */
      close(sockfd);
      if( (n=recv(newsockfd, recvline, MAXLINE,0)) < 0)
         exit(6);
      if( send(newsockfd, recvline,n,0) < n)
         exit(7);
      close(newsockfd);
      exit(0);
    }
    /*parent:*/
    close(newsockfd);
  }
}
