/* readline.c
 *
 * read a line from a stream socket descriptor
 * one byte at a time looking for the newline
 * newline is stored in the buffer follow it with a null
 * return value: number of characters up to, but not
 * including the null
 *
 * not very efficient but we don't need to worry about 
 * buffering in the Standard I/O Library 
 */

int readline(int fd, char *ptr, int maxlen) {
	int rc, n;
	char c;

	for (n=1; n < maxlen; n++) {
		if ( (rc = read(fd, &c, 1) ) == 1) {
			*ptr++ = c;
			if (c == '\n')
				break;
		} else if ( rc == 0 ) {
			if (n == 1)
				return (0); /*EOF, no data read*/
			else 
				break;      /*EOF, some data read*/
		} else
			return (-1);        /*error*/
	}
	*ptr = 0;
	return(n);
}
